#page {
	width: 95%;	
}


#browser {
	width: 25%;
	float: left;
}
#browser ul {
	margin: 0;
	padding: 0;
	list-style-type: none;
	font-size: .8em;
	margin-bottom: 1.5em;	
}

#browser h2 {
	text-transform: uppercase;
	font-size: 1em;
	margin: 0;
	margin-bottom: .5em;
	padding: 0;	
}


#main {
	width: 75%;
	float: left;
	padding-bottom: 2em;
}

#main h1 {
	font-weight: normal;
	letter-spacing: .1em;	
}

#main h2 {
	font-size: 1.1em;
	font-weight: normal;
}

#main h3 {
	font-size: 1em;
	clear: both;
}

#main p {
	text-align: justify;	
}


#main .field-list tbody {
	background-color: #eeeeec;
	font-size: .8em;
}
#main .field-name, .field-body {
	padding: .3em;
}
#main .field-list .field-body {
	background-color: white;
	padding-left: .3em;
}
#main .field-name {
	font-weight: normal;
	text-align: right;
	padding-right: .3em;
}

#main .link-bar {
	list-style-type: none;
	margin: 0;
	padding: 0;
	padding-bottom: 2em;
	margin-top: .3em;
	display: block;
	font-size: .7em;
}
#main .link-bar li {
	float: left;	
}
#main .link-bar li.top {
	float: right;	
}
#main .link-bar a {
	color: #aaaaaa;
	text-transform: uppercase;
}

.rst h1 {
	text-transform: uppercase;
	font-size: 1.1em;
	font-weight: normal;
	margin-bottom: .7em;
	margin-top: 1.5em;
}
.rst h1 a {
	color: #555753;
}
.rst .literal {
	font-size: 1.2em;
	color: #2e3436;
}
.rst dt {
	font-weight: bold;
}
