* { margin: 0; padding: 0; }
body {
	font-family: sans-serif;
	color: #555753;
	background-color: white;
	font-size: 11pt;	
}
p {
	margin-top: .5em;
	margin-bottom: .8em;
	text-align: justify;
	font-size: .9em;
	line-height: 1.8em;
}

dd {
	margin-left: 1.5em;
	margin-bottom: .8em;
	margin-top: .3em;	
}
ul {
	margin-left: 1.5em;
	margin-bottom: .5em;	
}

a {
	color: #4e9a06;
	text-decoration: none;	
}
a:hover {
	color: #5c7f91;	
}
a:visited {
	color: #4e9a06;	
}

#devref {
	font-family: Impact, Helvetica, sans-serif;
    text-align: left;
    font-size: 9pt;
    letter-spacing: .2em;
    color: #eeeeec;
    background-color: #555753;
    padding: .2em;
    padding-left: 20px;
	
}
#devref a { color: #eeeeec; text-decoration: none; }
#devref a:hover { color: #eeeeec; }
#devref a:after { content: " <<"; }

#page { margin: 20px; }

#header {
	border-bottom: 1px solid #babdb6;
	padding-bottom: .3em;
	margin-bottom: 1em;	
}

#logo {
	height: 75px;
	background: url(../img/crunchyfrog_logo_70x70.png) top left no-repeat;
	padding-left: 80px;
}
#appname {
	text-align: right;
	text-transform: uppercase;
	font-weight: normal;
	letter-spacing: .3em;
	font-size: 1.7em;
	margin-bottom: .3em;
	color: #2e3436;
}
#appname .emph { font-weight: bold; color: #555753; }
.slogan {
	display: block;
	font-size: 12pt;
	letter-spacing: .1em;
	float: right;
}

#footer {
	text-align: right;
	color: #babdb6;
	font-size: .8em;
	margin-top: 2em;
	border-top: 1px solid #eeeeec;
	padding-top: 1em;
	clear: both;
}

#content h2 {
	margin-top: 2em;
	margin-bottom: .5em;	
}

.skip { display: none; }
.clearfix { clear: both; }
.fr { float: right; }
ul.hmenu { list-style-type: none; }
ul.hmenu li { float: left; margin-left: 1em; }

#mainmenu { margin-top: .7em; }

#mainmenu a {
	text-decoration: none;
	font-size: .8em;
	text-transform: uppercase;
	color: #555753;
}
#mainmenu a:hover { color: #2e3436; }

.browser-help {
	font-size: .8em;
	text-transform: uppercase;
	color: #babdb6;
}

.child-elements h2 {
	font-weight: normal;
	text-transform: uppercase;	
}

.child-elements ul {
	list-style-type: none;
	margin: 0;
}
.child-element li {
	clear: both;
}
.child-elements a {
	display: block;
	padding: .2em;
	width: 100%;
	vertical-align:  baseline;
}
.child-elements li:hover {
	background-color: #eeeeec;	
}
.child-elements div.name {
	float: left;	
	width: 25%;
}
.child-elements div.doc {
	text-transform: uppercase;
	margin-top: .3em;
	color: #babdb6;
	font-size: .8em;
}

.even, .odd {
	border-bottom: 1px solid #eeeeec;
}

.odd {

}

.clearfix { clear: both; }

.source {
	border-left: 5px solid #eeeeec;
	padding-left: 1em;
}

.note, .warning {
	padding: 5px;
	padding-left: 45px;
	padding-top: 0px;
	font-size: .9em;
	border: 1px solid #eeeeec;
	-moz-border-radius: 8px;
}
.note { 
	background: url(../img/dialog-information.png) 5px 5px no-repeat #fffce8; 
}
.warning { 
	background: url(../img/dialog-warning.png) 5px 5px no-repeat #ffe4cc; 
}
.note .first, .warning .first { 
	text-transform: uppercase;
	letter-spacing: 0.25em; 
}

.breadcrumbs {
	m/argin-top: .5em;
	margin-bottom: 1.5em;
	padding-left: .5em;
	border-left: 3px solid #eeeeec;
}
.breadcrumbs .crumbs {
	font-family: Monospace; 
	float: left;
	font-size: .8em; 
}
.breadcrumbs .short_desc { 
	float: right;
	text-transform: uppercase;
	color: #babdb6;
}

a.reference {
	display: inline;
}